from pyuarm import *
from pyuarm.tools.list_uarms import get_uarm_port_cli
import copy
import argparse
import sys

INIT_POS_L = 139
INIT_POS_R = 26
SAMPLING_DEADZONE = 2


class Calibration(object):
    manual_calibration_trigger = True
    linear_calibration_start_flag = False
    stretch_calibration_flag = False

    linear_offset_template = {"INTERCEPT": 0.00, "SLOPE": 0.00}
    linear_offset = [linear_offset_template, linear_offset_template, linear_offset_template, linear_offset_template]
    linear_offset_servo_flag = [False, False, False, False]
    manual_offset = [0.00, 0.00, 0.00]
    stretch_offset_template = {"LEFT": 0.00, "RIGHT": 0.00}
    temp_manual_offset_arr = [0.00, 0.00, 0.00, 0.00]
    manual_offset_correct_flag = [False, False, False, False]

    def __init__(self, uarm, log_function=None):
        if uarm is not None:
            self.uarm = uarm
        else:
            raise pyuarm.NoUArmPortException("No uArm Port provided")
        if log_function is not None:
            self.log_function = log_function
        else:
            self.log_function = self.default_log
        self.servo_calibrate_timeout = 300
        self.is_all_calibrated = False
        self.is_linear_calibrated = False
        self.is_manual_calibrated = False
        self.is_stretch_calibrated = False

    @staticmethod
    def default_log(msg):
        print msg

    def uf_print(self, msg=None):
        self.log_function(msg)

    def calibrate_all(self, linear_callback=None, manual_callback=None, stretch_callback=None):
        self.uf_print("0. Clearing Completed Flag in EEPROM.")
        self.write_completed_flag(CALIBRATION_FLAG, False)
        time.sleep(1)

        if linear_callback is None:  # If not provide callback function use default
            self.linear_calibration_section(None)
        else:
            linear_callback()

        if self.read_completed_flag(CALIBRATION_LINEAR_FLAG):
            if manual_callback is None:  # If not provide callback function use default
                self.manual_calibration_section(None)
            else:
                manual_callback()
            time.sleep(1)

        if self.read_completed_flag(CALIBRATION_SERVO_FLAG):
            # if stretch_callback is None:  # If not provide callback function use default
            self.stretch_calibration_section()
            # else:
            #     stretch_callback()
            time.sleep(1)

        if self.read_completed_flag(CALIBRATION_STRETCH_FLAG):
            self.write_completed_flag(CALIBRATION_FLAG, True)
            self.uf_print("Calibration DONE!!")
            self.uarm.attach_all_servos()
            self.uarm.write_servo_angle(SERVO_BOTTOM,90)
            self.uarm.write_left_right_servo_angle(60,60,True)
            self.uarm.write_servo_angle(SERVO_HAND,90, True)

    def linear_calibration_section(self, callback=None):
        self.linear_calibration_start_flag = True
        self.uf_print("1.0. Clearing Linear Completed Flag in EEPROM.")
        self.write_completed_flag(CALIBRATION_LINEAR_FLAG, False)
        self.uf_print("1. Start Calibrate Linear Offset")
        for i in range(4):
            self.uf_print("    1.1. {0} Linear Offset - Servo {1}".format(str(i), str(i)))
            temp_linear_offset = self.calibrate_linear_servo_offset(i)
            linear_is_correct = False
            if check_linear_is_correct(temp_linear_offset):
                self.linear_offset[i] = temp_linear_offset
                linear_is_correct = True
            else:
                linear_is_correct = False
            if callback is not None:
                callback(i, linear_is_correct, temp_linear_offset)

        self.save_linear_offset()
        time.sleep(1)
        if self.read_linear_offset() == self.linear_offset:
            self.uf_print("    1.3 Mark Completed Flag in EEPROM")
            self.write_completed_flag(CALIBRATION_LINEAR_FLAG, True)
            self.linear_calibration_start_flag = False
        else:
            self.uf_print("Error - 1. Linear Offset not equal to EEPROM, Please retry.")
            self.uf_print("Error - 1. linear_offset(): {0}".format(self.read_linear_offset()))
            self.linear_calibration_start_flag = False

    def calibrate_linear_servo_offset(self, servo_number):
        global analog_read_pin, servo_analog_read
        angles = []
        analogs = []
        self.ab_values = []

        if servo_number == SERVO_BOTTOM:
            max_angle = 150
            min_angle = 30
            angle_step = min_angle
            analog_pin = SERVO_BOTTOM_ANALOG_PIN
            self.uarm.write_servo_angle(SERVO_BOTTOM, angle_step, False)
            self.uarm.write_servo_angle(SERVO_LEFT, 90, False)
            self.uarm.write_servo_angle(SERVO_RIGHT, 60, False)
        if servo_number == SERVO_LEFT:
            max_angle = 120
            min_angle = 35
            angle_step = min_angle
            analog_pin = SERVO_LEFT_ANALOG_PIN
            self.uarm.write_servo_angle(SERVO_BOTTOM, 90, False)
            self.uarm.write_servo_angle(SERVO_LEFT, angle_step, False)
            self.uarm.write_servo_angle(SERVO_RIGHT, 65, False)
        if servo_number == SERVO_RIGHT:
            max_angle = 120
            min_angle = 15
            angle_step = min_angle
            analog_pin = SERVO_RIGHT_ANALOG_PIN
            self.uarm.write_servo_angle(SERVO_BOTTOM, 90, False)
            self.uarm.write_servo_angle(SERVO_LEFT, 65, False)
            self.uarm.write_servo_angle(SERVO_RIGHT, angle_step, False)
        if servo_number == SERVO_HAND:
            max_angle = 160
            min_angle = 20
            angle_step = min_angle
            analog_pin = SERVO_HAND_ANALOG_PIN
            self.uarm.write_servo_angle(SERVO_BOTTOM, 90, False)
            self.uarm.write_servo_angle(SERVO_LEFT, 90, False)
            self.uarm.write_servo_angle(SERVO_RIGHT, 60, False)
            self.uarm.write_servo_angle(SERVO_HAND, angle_step, False)
        time.sleep(2)
        while angle_step < max_angle:

            self.uarm.write_servo_angle(servo_number, angle_step, False)

            servo_analog_read = 0
            for i in range(5):
                servo_analog_read += self.uarm.read_analog(analog_pin)
                time.sleep(0.05)

            servo_analog_read /= 5
            angles.append(angle_step)
            analogs.append(servo_analog_read)
            self.uf_print ("Servo Number: {0}, Angle: {1}, Analog: {2}".format(servo_number, angle_step, servo_analog_read))
            angle_step += 1
            time.sleep(0.1)

        new_ab = basic_linear_regression(analogs, angles)

        linear_offset_template = copy.deepcopy(self.linear_offset_template)
        linear_offset_template['SLOPE'] = round(new_ab[0], 2)
        linear_offset_template['INTERCEPT'] = round(new_ab[1], 2)
        return linear_offset_template

    def manual_calibration_section(self, callback=None):
        self.uf_print("2.0. Clearing Servo Completed Flag in EEPROM.")
        self.write_completed_flag(CALIBRATION_SERVO_FLAG, False)
        self.uf_print("2. Start Calibrate Servo Offset")
        self.manual_operation_trigger = True
        self.uarm.write_servo_angle(SERVO_BOTTOM, 45, 0)
        time.sleep(1)
        self.uarm.write_left_right_servo_angle(130, 20, 0)
        time.sleep(1)
        self.uarm.write_servo_angle(SERVO_HAND, 90, 0)
        time.sleep(1)
        self.uarm.detach_all_servos()
        time.sleep(0.3)

        time_counts = 0

        self.servo_1_offset = 0
        self.servo_2_offset = 0
        self.servo_3_offset = 0
        self.uf_print("Please move uArm in right position")
        self.uarm.alarm(3, 100, 100)
        while self.manual_operation_trigger:


                # self.uf_print("Please try move the Servo 3")

            # if time_counts > self.servo_calibrate_timeout:
            #     self.manual_operation_trigger = False
            self.__read_current_angle()
            time_counts += 1
            self.temp_manual_offset_arr[0] = self.servo_1_offset
            self.temp_manual_offset_arr[1] = self.servo_2_offset
            self.temp_manual_offset_arr[2] = self.servo_3_offset
            self.temp_manual_offset_arr[3] = self.servo_4_offset
            # if self.manual_offset_correct_flag[0] & self.manual_offset_correct_flag[1] & \
            #         self.manual_offset_correct_flag[2]:
            #     self.uf_print(str(self.servo_calibrate_timeout - time_counts) + ", Please Confirm the positions")
            if callback is not None:
                callback(self.temp_manual_offset_arr, self.manual_offset_correct_flag)
                self.uf_print()
            else:
                confirm_enter = raw_input("Please move uArm to correct position, When complete, please press Enter to contine. ")
                if confirm_enter == "":
                    confirm = ""
                    while confirm.lower() != "y":
                        self.__read_current_angle()
                        confirm = raw_input(
                            "servo offset, bottom: {0}, left: {1}, right: {2}, hand: {3},\nConfirm Please Press Y, Retry press Other Key: "
                                .format(self.servo_1_offset,self.servo_2_offset,self.servo_3_offset, self.servo_4_offset))
                        if confirm.lower() == "y":
                            self.manual_operation_trigger = False
            time.sleep(0.1)

        self.temp_manual_offset_arr[0] = round(self.servo_1_offset, 2)
        self.temp_manual_offset_arr[1] = round(self.servo_2_offset, 2)
        self.temp_manual_offset_arr[2] = round(self.servo_3_offset, 2)
        self.temp_manual_offset_arr[3] = round(self.servo_4_offset, 2)
        self.save_manual_offset()
        if self.read_manual_offset() == self.temp_manual_offset_arr:
            self.uf_print("    2.3 Mark Completed Flag in EEPROM")
            self.write_completed_flag(CALIBRATION_SERVO_FLAG, True)
        else:
            self.write_completed_flag(CALIBRATION_SERVO_FLAG, False)
            self.uf_print("Error - 2, Manual Calibration Servo offset not equal to EEPROM")
            self.uf_print("Error - 2, manual Servo Offset: {0}".format(self.temp_manual_offset_arr))
            self.uf_print("Error - 2, read_manual_offset: {0}".format(self.read_manual_offset()))
        self.uarm.attach_all_servos()
    
    def __read_current_angle(self):
        self.servo_1_offset = self.uarm.read_servo_angle(SERVO_BOTTOM, 0) - 45
        self.servo_2_offset = self.uarm.read_servo_angle(SERVO_LEFT, 0) - 130
        self.servo_3_offset = self.uarm.read_servo_angle(SERVO_RIGHT, 0) - 20
        self.servo_4_offset = self.uarm.read_servo_angle(SERVO_HAND, 0) - 90

        if abs(self.servo_1_offset) < 5.5:
            self.manual_offset_correct_flag[0] = True
        else:
            self.manual_offset_correct_flag[0] = False
            # self.uf_print("Please try move the Servo 1")
        if abs(self.servo_2_offset) < 5.5:
            self.manual_offset_correct_flag[1] = True
        else:
            self.manual_offset_correct_flag[1] = False
            # self.uf_print("Please try move the Servo 2")
        if abs(self.servo_3_offset) < 5.5:
            self.manual_offset_correct_flag[2] = True
        else:
            self.manual_offset_correct_flag[2] = False
        if abs(self.servo_4_offset) < 5.5:
            self.manual_offset_correct_flag[3] = True
        else:
            self.manual_offset_correct_flag[3] = False

    def stretch_calibration_section(self):
        self.stretch_calibration_flag = True
        self.uf_print("3.0. Clearing Stretch Completed Flag in EEPROM.")
        self.write_completed_flag(CALIBRATION_STRETCH_FLAG, False)
        self.uf_print("    3.1 Saving Stretch Offset into EEPROM")
        self.uarm.write_eeprom(EEPROM_DATA_TYPE_FLOAT, OFFSET_STRETCH_START_ADDRESS, -10)
        self.uarm.write_eeprom(EEPROM_DATA_TYPE_FLOAT, OFFSET_STRETCH_START_ADDRESS + 4, -10)
        self.uf_print("    3.2 Mark Completed Flag in EEPROM")
        self.write_completed_flag(CALIBRATION_STRETCH_FLAG, True)
        # print offsetL, offsetR
        self.uarm.detach_all_servos()
        self.stretch_calibration_flag = False

    def save_linear_offset(self):
        self.uf_print("    1.2 Saving Servo Offset into EEPROM")
        intercept_address = LINEAR_INTERCEPT_START_ADDRESS
        slope_address = LINEAR_SLOPE_START_ADDRESS
        for i in range(4):
            print ("Intercept Address:{0}, Offset Value:{1}.".format(intercept_address, self.linear_offset[i]['INTERCEPT']))
            print ("Slope Address:{0}, Offset Value:{1}.".format(slope_address, self.linear_offset[i]['SLOPE']))
            self.uarm.write_eeprom(EEPROM_DATA_TYPE_FLOAT, intercept_address, self.linear_offset[i]['INTERCEPT'])
            time.sleep(0.5)
            self.uarm.write_eeprom(EEPROM_DATA_TYPE_FLOAT, slope_address, self.linear_offset[i]['SLOPE'])
            time.sleep(0.5)
            intercept_address += 4
            slope_address += 4

    def save_manual_offset(self):
        self.uf_print("    2.1 Saving Servo Offset into EEPROM")
        address = OFFSET_START_ADDRESS
        for i in self.temp_manual_offset_arr:
            self.uarm.write_eeprom(EEPROM_DATA_TYPE_FLOAT, address, i)
            address += 4

    def read_manual_offset(self):
        address = OFFSET_START_ADDRESS
        read_manual_offset = []
        for i in range(4):
            read_manual_offset.append(round(self.uarm.read_eeprom(EEPROM_DATA_TYPE_FLOAT, address), 2))
            address += 4
        return read_manual_offset

    def read_linear_offset(self):
        intercept_address = LINEAR_INTERCEPT_START_ADDRESS
        slope_address = LINEAR_SLOPE_START_ADDRESS
        linear_offset_data = []
        for i in range(4):
            linear_offset_template = copy.deepcopy(self.linear_offset_template)
            linear_offset_template['INTERCEPT'] = round(
                self.uarm.read_eeprom(EEPROM_DATA_TYPE_FLOAT, intercept_address), 2)
            linear_offset_template['SLOPE'] = round(self.uarm.read_eeprom(EEPROM_DATA_TYPE_FLOAT, slope_address), 2)
            linear_offset_data.append(linear_offset_template)
            intercept_address += 4
            slope_address += 4
        return linear_offset_data

    def read_stretch_offset(self):
        address = OFFSET_STRETCH_START_ADDRESS
        left_offset = round(self.uarm.read_eeprom(EEPROM_DATA_TYPE_FLOAT, address), 2)
        right_offset = round(self.uarm.read_eeprom(EEPROM_DATA_TYPE_FLOAT, address + 4), 2)
        stretch_offset = copy.deepcopy(self.stretch_offset_template)
        stretch_offset['LEFT'] = left_offset
        stretch_offset['RIGHT'] = right_offset
        return stretch_offset

    def init_calibration_completed_flag(self):
        self.is_all_calibrated = self.read_completed_flag(CALIBRATION_FLAG)
        self.is_linear_calibrated = self.read_completed_flag(CALIBRATION_LINEAR_FLAG)
        self.is_manual_calibrated = self.read_completed_flag(CALIBRATION_SERVO_FLAG)
        self.is_stretch_calibrated = self.read_completed_flag(CALIBRATION_STRETCH_FLAG)

    def write_completed_flag(self, flag_type, flag):
        if flag_type == CALIBRATION_FLAG:
            self.uarm.write_eeprom(EEPROM_DATA_TYPE_BYTE, CALIBRATION_FLAG, CONFIRM_FLAG if flag else 0)
        elif flag_type == CALIBRATION_LINEAR_FLAG:
            self.uarm.write_eeprom(EEPROM_DATA_TYPE_BYTE, CALIBRATION_LINEAR_FLAG, CONFIRM_FLAG if flag else 0)
        elif flag_type == CALIBRATION_SERVO_FLAG:
            self.uarm.write_eeprom(EEPROM_DATA_TYPE_BYTE, CALIBRATION_SERVO_FLAG, CONFIRM_FLAG if flag else 0)
        elif flag_type == CALIBRATION_STRETCH_FLAG:
            self.uarm.write_eeprom(EEPROM_DATA_TYPE_BYTE, CALIBRATION_STRETCH_FLAG, CONFIRM_FLAG if flag else 0)

    def read_completed_flag(self, flag_type):
        if flag_type == CALIBRATION_FLAG:
            if self.uarm.read_eeprom(EEPROM_DATA_TYPE_BYTE, CALIBRATION_FLAG) == CONFIRM_FLAG:
                return True
            else:
                return False
        elif flag_type == CALIBRATION_LINEAR_FLAG:
            if self.uarm.read_eeprom(EEPROM_DATA_TYPE_BYTE, CALIBRATION_LINEAR_FLAG) == CONFIRM_FLAG:
                return True
            else:
                return False
        elif flag_type == CALIBRATION_SERVO_FLAG:
            if self.uarm.read_eeprom(EEPROM_DATA_TYPE_BYTE, CALIBRATION_SERVO_FLAG) == CONFIRM_FLAG:
                return True
            else:
                return False
        elif flag_type == CALIBRATION_STRETCH_FLAG:
            if self.uarm.read_eeprom(EEPROM_DATA_TYPE_BYTE, CALIBRATION_STRETCH_FLAG) == CONFIRM_FLAG:
                return True
            else:
                return False


def basic_linear_regression(x, y):
    def float_data(a):
        for i in range(len(a)):
            a[i] = float(a[i])
        return a

    x = float_data(x)
    y = float_data(y)

    # Basic computations to save a little time.
    length = len(x)
    sum_x = sum(x)
    sum_y = sum(y)

    sum_x_squared = sum(map(lambda a: a * a, x))
    sum_of_products = sum([x[i] * y[i] for i in range(length)])

    # Magic formulae!
    a = (sum_of_products - (sum_x * sum_y) / length) / (sum_x_squared - ((sum_x ** 2) / length))
    b = (sum_y - a * sum_x) / length
    return a, b


def check_linear_is_correct(linear_offset):
    if linear_offset['SLOPE'] > 0.1 and linear_offset['INTERCEPT'] < 1:
        return True


def main():
    """
    uArm Calibration Library
        1. Linear Calibration section
          1.1 Write 20 times Angle to each Servo, in the meanwhile read the 20 times Servo Analog
          1.2 Using basic_linear_regression(analogs, angles) to generate the intercept & slope
              ``(In arduino library, angle = analog * INTERCEPT + SLOPE)``
          1.3 Save each servo linear offset in EEPROM (LINEAR_INTERCEPT_START_ADDRESS, LINEAR_SLOPE_START_ADDRESS)
          1.4 Mark Linear Calibration in EEPROM (CALIBRATION_LINEAR_FLAG)
         2. Manual Calibration section
          2.1 Set uArm at standard Angles. Left Angle - 130, Right Angle 30, Bottom Angle, 45.
          2.2 Put uArm into the calibration position.
          2.3 Confirm the Position.
         3. Stretch Calibration section
    """
    parser = argparse.ArgumentParser()
    parser.add_argument("-f", "--force", help="Force calibrate uArm, `all` , `linear`, `manual`")
    parser.add_argument("-c", "--check", help="Check If Calibration is completed? if completed, display the offset value", action="store_true")
    parser.add_argument("-p", "--port", nargs='?', help="specify port number")
    args = parser.parse_args()
    if args.port:
        calibration = Calibration(pyuarm.uArm(port=args.port))
    else:
        calibration = Calibration(pyuarm.get_uarm())
    time.sleep(2)
    # check
    if args.check:
        print ("\n")
        print ("-------------------------------------------------")
        print ("Checking Calibration information....")
        print ("-------------------------------------------------")
        is_linear_calibrated = calibration.read_completed_flag(CALIBRATION_LINEAR_FLAG)
        is_manual_calibrated = calibration.read_completed_flag(CALIBRATION_SERVO_FLAG)
        is_all_calibrated = calibration.read_completed_flag(CALIBRATION_FLAG)
        if is_linear_calibrated and is_manual_calibrated and is_all_calibrated:
            print ("Calibration All Completed !!!")
            linear_offset = calibration.read_linear_offset()
            print("Linear Offset:\n")
            print ("Bottom Servo, INTERCEPT: {0}, SLOPE: {1}".format(linear_offset[0]['INTERCEPT'],linear_offset[0]['SLOPE']))
            print ("Left Servo, INTERCEPT: {0}, SLOPE: {1}".format(linear_offset[1]['INTERCEPT'],linear_offset[1]['SLOPE']))
            print ("Right Servo, INTERCEPT: {0}, SLOPE: {1}".format(linear_offset[2]['INTERCEPT'],linear_offset[2]['SLOPE']))
            print ("Hand Servo, INTERCEPT: {0}, SLOPE: {1}".format(linear_offset[3]['INTERCEPT'],linear_offset[3]['SLOPE']))
            manual_offset = calibration.read_manual_offset()
            print("Manual Offset:\n")
            print ("Bottom Servo Offset: {0}".format(manual_offset[0]))
            print ("Left Servo Offset: {0}".format(manual_offset[1]))
            print ("Right Servo Offset: {0}".format(manual_offset[2]))
        elif is_linear_calibrated and is_manual_calibrated and not is_all_calibrated:
            print ("Calibration All Completed !!!")
        elif not is_linear_calibrated and is_manual_calibrated:
            print ("Linear Calibration not Completed !!!")
        elif is_linear_calibrated and not is_manual_calibrated:
            print ("Manual Calibration not Completed !!!")
        elif not is_linear_calibrated and not is_manual_calibrated:
            print ("Linear Calibration not Completed !!!")
            print ("Manual Calibration not Completed !!!")
        exit_fun()
    if args.force:
        print ("\n")
        print ("-------------------------------------------------")
        print ("Force Calibrating....")
        print ("-------------------------------------------------")
        if args.force == "all":
            calibration.calibrate_all()
        elif args.force == "linear":
            calibration.linear_calibration_section()
        elif args.force == "manual":
            if calibration.read_completed_flag(CALIBRATION_LINEAR_FLAG):
                calibration.manual_calibration_section()
            else:
                print ("Please Complete Linear Calibration First! You could use -f linear")
        else:
            print ("unrecognized command: {0}".format(args.force))
        exit_fun()

    # No Argument
    is_linear_calibrated = calibration.read_completed_flag(CALIBRATION_LINEAR_FLAG)
    is_manual_calibrated = calibration.read_completed_flag(CALIBRATION_SERVO_FLAG)
    is_all_calibrated = calibration.read_completed_flag(CALIBRATION_FLAG)
    if is_linear_calibrated and is_manual_calibrated and is_all_calibrated:
        print ("uArm has been calibrated already, Are you sure want to Calibrate it again?")
        confirm = raw_input("Press Y if you want to calibrate anyway...\n")
        if confirm == "Y" or confirm=="y":
            calibration.calibrate_all()
        else:
            exit_fun()
    else:
        calibration.calibrate_all()
    exit_fun()


def exit_fun():
    raw_input("\nPress Enter to Exit...")
    sys.exit(0)


if __name__ == '__main__':
    try:
        main()
    except SystemExit:
        exit_fun()



#!/usr/bin/env python
#
# This is a module that help user to manage the uArm firmware.
# Please use -h to list all parameters of this script

# This file is part of pyuarm. https://github.com/uArm-Developer/pyuarm
# (C) 2016 UFACTORY <developer@ufactory.cc>

import pyuarm
from pyuarm.tools.list_uarms import get_uarm_port_cli
from progressbar import ProgressBar, Percentage, FileTransferSpeed, Bar, ETA
import requests
import os, sys, platform, subprocess
from itertools import izip
import ConfigParser, StringIO

from distutils.version import LooseVersion
import argparse

firmware_download_url = "http://download.ufactory.cc/firmware.hex"
remote_version_url = "http://download.ufactory.cc/version"


if getattr(sys, 'frozen', False):
    application_path = os.path.dirname(sys.executable)
elif __file__:
    application_path = os.path.dirname(__file__)

firmware_defaul_filename = 'firmware.hex'
default_firmware_path = os.path.join(os.getcwd(), firmware_defaul_filename)



    # if len(uarm_list) > 0:
    #     return uarm_list[0]
    # else:
    #     print "No uArm is connected."


def get_latest_version(release_url=remote_version_url):
    print ("Fetching the latest firmware version...")
    try:
        r = requests.get(release_url)
        s_config = r.text
        buf = StringIO.StringIO(s_config)
        config = ConfigParser.ConfigParser()
        config.readfp(buf)
        remote_firmware_version = config.get('firmware', 'version')
        return remote_firmware_version
        # print("Remote Version: {0}".format(self.remote_firmware_version))
    except requests.exceptions.ConnectionError:
        raise NetworkError("NetWork Error, Please retry...")


def download_firmware(firmware_path=default_firmware_path, firmware_url=firmware_download_url):
    print ("Downloading firmware.hex...")
    try:
        response = requests.get(firmware_url, stream=True)
        firmware_size = int(response.headers['content-length'])
        with open(firmware_path, "wb") as handle:
            widgets = ['Downloading: ', Percentage(), ' ',
                       Bar(marker='#', left='[', right=']'),
                       ' ', ETA(), ' ', FileTransferSpeed()]
            pbar = ProgressBar(widgets=widgets, maxval=firmware_size)
            pbar.start()
            for i, data in izip(range(firmware_size), response.iter_content()):
                handle.write(data)
                pbar.update(i)
            pbar.finish()
    except requests.exceptions.ConnectionError:
        raise NetworkError("NetWork Error, Please retry...")


def flash_firmware(self,firmware_path='firmware.hex'):
    port = self.uarm_port
    if port:
        global avrdude_path, error_description, cmd
        port_conf = '-P' + port
        if platform.system() == 'Darwin':
            cmd = ['avrdude', '-v', '-patmega328p', '-carduino', port_conf, '-b115200', '-D',
               '-Uflash:w:{0}:i'.format(firmware_path)]
            error_description = "avrdude is required, Trying to install avrdude..."

        elif platform.system() == 'Windows':
            exe_name = 'avrdude.exe'
            conf_name = 'avrdude.conf'
            avrdude_path = os.path.join(application_path, 'avrdude', exe_name)
            avrdude_conf = os.path.join(application_path, 'avrdude', conf_name)
            cmd = [avrdude_path, '-C' + avrdude_conf, '-v', '-patmega328p', '-carduino', port_conf, '-b115200', '-D',
               '-Uflash:w:{0}:i'.format(firmware_path)]
            error_description = "avrdude is required, Please try install avrdude."

        elif platform.system() == 'Linux':
            cmd = ['avrdude', '-v', '-patmega328p', '-carduino', port_conf, '-b115200', '-D',
               '-Uflash:w:{0}:i'.format(firmware_path)]
            error_description = "avrdude is required, Trying to install avrdude"

        print (' '.join(cmd))
        try:
            subprocess.call(cmd)
        except OSError as e:
            print ("Error occurred: error code {0}, error msg: {1}".format(str(e.errno), e.strerror))
            if e.errno == 2:
                if platform.system() == 'Darwin':
                    try:
                        print ("Installing avrdude...")
                        subprocess.call(['brew', 'install', 'avrdude'])
                        subprocess.call(cmd)
                    except OSError as e:
                        print ("Error occurred: error code {0}, error msg: {1}".format(str(e.errno), e.strerror))
                        if e.errno == 2:
                            print ("-------------------------------------------------------")
                            print ("You didn't install homebrew, please visit http://bew.sh")
                            print ("-------------------------------------------------------")
                if platform.system() == 'Linux':
                    print ("------------------------------------------------------------------------------")
                    print ("You didn't install avrdude.\n "
                           "please try `sudo apt-get install avrdude` or other package management command ")
                    print ("------------------------------------------------------------------------------")


class FirmwareHelper():

    uarm_firmware_version = ""
    uarm_port = None

    def __init__(self, port=None):
        if port is not None:
            self.uarm_port = port
        else:
            self.uarm_port = get_uarm_port_cli()
        self.remote_firmware_version = "0.0.0"
        self.uarm_firmware_version = "0.0.0"

    def flash_firmware(self,firmware_path='firmware.hex'):
        port = self.uarm_port
        if port:
            global avrdude_path, error_description, cmd
            port_conf = '-P' + port
            if platform.system() == 'Darwin':
                cmd = ['avrdude', '-v', '-patmega328p', '-carduino', port_conf, '-b115200', '-D',
                   '-Uflash:w:{0}:i'.format(firmware_path)]
                error_description = "avrdude is required, Trying to install avrdude..."

            elif platform.system() == 'Windows':
                exe_name = 'avrdude.exe'
                conf_name = 'avrdude.conf'
                avrdude_path = os.path.join(application_path, 'avrdude', exe_name)
                avrdude_conf = os.path.join(application_path, 'avrdude', conf_name)
                cmd = [avrdude_path, '-C' + avrdude_conf, '-v', '-patmega328p', '-carduino', port_conf, '-b115200', '-D',
                   '-Uflash:w:{0}:i'.format(firmware_path)]
                error_description = "avrdude is required, Please try install avrdude."

            elif platform.system() == 'Linux':
                cmd = ['avrdude', '-v', '-patmega328p', '-carduino', port_conf, '-b115200', '-D',
                   '-Uflash:w:{0}:i'.format(firmware_path)]
                error_description = "avrdude is required, Trying to install avrdude"

            print (' '.join(cmd))
            try:
                subprocess.call(cmd)
            except OSError as e:
                print ("Error occurred: error code {0}, error msg: {1}".format(str(e.errno), e.strerror))
                if e.errno == 2:
                    if platform.system() == 'Darwin':
                        try:
                            print ("Installing avrdude...")
                            subprocess.call(['brew', 'install', 'avrdude'])
                            subprocess.call(cmd)
                        except OSError as e:
                            print ("Error occurred: error code {0}, error msg: {1}".format(str(e.errno), e.strerror))
                            if e.errno == 2:
                                print ("-------------------------------------------------------")
                                print ("You didn't install homebrew, please visit http://bew.sh")
                                print ("-------------------------------------------------------")
                    if platform.system() == 'Linux':
                        print ("------------------------------------------------------------------------------")
                        print ("You didn't install avrdude.\n "
                               "please try `sudo apt-get install avrdude` or other package management command ")
                        print ("------------------------------------------------------------------------------")
    def get_uarm_version(self):
        if self.uarm_port is not None or self.uarm_port != "":
            print ("Reading Firmware version from uArm...")
            try:
                uarm = pyuarm.uArm(port=self.uarm_port)
                self.uarm_firmware_version = uarm.firmware_version
                uarm.disconnect()
                return self.uarm_firmware_version
            except pyuarm.UnknownFirmwareException:
                print "Unknown Firmware version."
            except pyuarm.NoUArmPortException:
                print "No uArm is connected."

    def compare_version(self):
        self.remote_firmware_version = get_latest_version()
        if  LooseVersion(self.get_uarm_version()) < LooseVersion(get_latest_version()):
            return True

    def upgrade(self):
        try:
            if self.compare_version():
                print ("Would you want to upgrade your uArm with {0}{1}".format(self.remote_firmware_version, "?"))
                user_choice = raw_input("Please Enter Y if yes. ")
                if user_choice == "Y" or user_choice == "y":
                    download_firmware()
                    self.flash_firmware()

                else:
                    print ("Exit")
            else:
                print("You already have the latest version of Firmware installed in uArm!!!")
                print("If you still want to download & flash the firmware, Please use `uarm-firmware -df`")
        except TypeError as e:
            print "Latest Firmware version: {0} ".format(self.remote_firmware_version)
            print ("Unknown uArm Firmware version, Would you want to upgrade your uArm with {0}{1}".format(self.remote_firmware_version, "?"))
            user_choice = raw_input("Please Enter Y if yes. ")
            if user_choice == "Y" or user_choice == "y":
                try:
                    download_firmware()
                    self.flash_firmware()
                except NetworkError as e:
                    print (e.error)
                except APIError as e:
                    print (e.error)
            else:
                print ("Exit")
        except NetworkError as e:
            print (e.error)
        except APIError as e:
            print (e.error)


def main():
    """
    ::

        $ uarm-firmware -h

        usage: firmware_helper [-h] [-d] [-f [FORCE]] [-c [CHECK]] [-p [PORT]]

        optional arguments:
          -h, --help            show this help message and exit
          -d, --download        download firmware into firmware.hex
          -f [FORCE], --force [FORCE]
                                without firmware path, flash default firmware.hex,
                                with firmware path, flash the firmware, eg. -f
                                Blink.ino.hex
          -c [CHECK], --check [CHECK]
                                remote - lateset firmware release version, local -
                                read uArm firmware version
          -p [PORT], --port [PORT]
                                provide port number
          -u, --upgrade         Upgrade firmware if remote version newer than local
                                version
    """

    parser = argparse.ArgumentParser()
    parser.add_argument("-d", "--download", help="download firmware into firmware.hex", action="store_true")
    parser.add_argument("-f", "--force", nargs='?', const='force',
                        help="without firmware path, flash default firmware.hex, with firmware path, flash the firmware, eg. -f Blink.ino.hex")
    parser.add_argument("-c", "--check", nargs='?', const='local',
                        help="remote - lateset firmware release version, local - read uArm firmware version")
    parser.add_argument("-p", "--port", nargs='?', help="provide port number")
    parser.add_argument("-u", "--upgrade", help="Upgrade firmware if remote version newer than local version", action="store_true")
    args = parser.parse_args()
    helper = None
    if args.force or args.upgrade or args.check:
        if args.port:
            helper = FirmwareHelper(args.port)
        else:
            helper = FirmwareHelper()

    # download
    if args.download:
        try:
            download_firmware()
        except NetworkError as e:
            print (e.error)
        except APIError as e:
            print (e.error)
    # force
    if args.force == "force":
        if not os.path.exists(default_firmware_path):
            print "firmware.hex not existed"
        else:
            if args.port:
                helper.flash_firmware()
            else:
                helper.flash_firmware()
        sys.exit(0)
    elif args.force is not None and args.force != "":
        if not os.path.exists(args.force):
            print args.force + " not existed."
        else:
            if args.port:
                helper.flash_firmware(firmware_path=args.force)
            else:
                helper.flash_firmware(firmware_path=args.force)
            sys.exit(0)
    # check
    if args.check == "local":
        if helper.uarm_port is None or helper.uarm_port == "":
            sys.exit(0)
        helper.get_uarm_version()
        sys.exit(0)
    elif args.check == "remote":
        print "Fetching the remote version..."
        remote_firmware_version = get_latest_version()
        print "Latest firmware release version is: {0}".format(remote_firmware_version)
        sys.exit(0)

    # upgrade
    if args.upgrade:
        helper.upgrade()
        sys.exit(0)

    if helper is not None:
        if helper.uarm_port is None or helper.uarm_port == "":
            sys.exit(0)
    else:
        sys.exit(0)

    if not args.download:
        helper.upgrade()
        sys.exit(0)


def exit_fun():
    raw_input("\nPress Enter to Exit...")
    sys.exit(0)


class NetworkError(Exception):
    def __init__(self, error):
        self.error = error

    def __str__(self):
        return repr(self.error)


class APIError(Exception):
    def __init__(self, error):
        self.error = error

    def __str__(self):
        return repr(self.error)


if __name__ == '__main__':
    try:
        main()
    except SystemExit:
        exit_fun()



#!/usr/bin/env python
#
# This is a module that help user to control your uArm.
# Please use -h to list all parameters of this script

# This file is part of pyuarm. https://github.com/uArm-Developer/pyuarm
# (C) 2016 UFACTORY <developer@ufactory.cc>


import pyuarm
from cmd import Cmd
from pyuarm.tools.list_uarms import get_uarm_port_cli ,uarm_ports
from pyuarm.tools import firmware_helper
import subprocess
from colorama import Fore, Back, init, Style

# logging.basicConfig(filename='logger.log', level=logging.INFO)
#
# logging.info("------------------------------------")
#

# class bcolors:
#     HEADER = '\033[95m'
#     OKBLUE = '\033[94m'
#     OKGREEN = '\033[92m'
#     WARNING = '\033[93m'
#     FAIL = '\033[91m'
#     ENDC = '\033[0m'
#     BOLD = '\033[1m'
#     UNDERLINE = '\033[4m'

init()


class UArmCmd(Cmd):

    help_msg = Style.BRIGHT + Fore.MAGENTA + "Shortcut:" + Fore.RESET + Style.RESET_ALL + "\n"
    help_msg += "Quit: " + Style.BRIGHT + Fore.RED + "Ctrl + D" + Fore.RESET + Style.RESET_ALL
    help_msg += ", or input: " + Back.BLACK + Fore.WHITE + "quit" + Fore.RESET + Style.RESET_ALL + "\n"
    help_msg += "Clear Screen: " + Style.BRIGHT + Fore.RED + "Ctrl + L" + Fore.RESET + Style.RESET_ALL

    ON_OFF = ['on', 'off']

    FIRMWARE =['version', 'force', 'upgrade']

    SERVO_STATUS = ['attach', 'detach']

    prompt = ">>> "
    intro = "Welcome to use {}uArm Command Line{}\n"\
        .format(Fore.YELLOW, Fore.RESET)

    intro += help_msg
    intro += "\n\n"
    intro += "Input {}{}help{}{} for more usage".format(Back.BLACK, Fore.WHITE,Fore.RESET, Back.RESET)

    # doc_header = "documented commands:"
    ruler = '-'

    uarm = None

    def __init__(self,  *args, **kwargs):
        Cmd.__init__(self, *args, **kwargs)

    def __is_connected(self):
        if self.uarm is None:
            print ("No uArm is connected, please use {}{}connect{}{}".format(Back.BLACK, Fore.WHITE,Fore.RESET, Back.RESET))
            return False
        else:
            if self.uarm.is_connected():
                return True
            else:
                print ("No uArm is connected, please use {}{}connect{}{}".format(Back.BLACK, Fore.WHITE,Fore.RESET, Back.RESET))
                return False

    def do_connect(self, arg):
        """
        connect, Open uArm Port, if more than one uArm Port found, will prompt options to choose.
        Please connect to uArm before you do any control action
        """
        if self.uarm is None:
            ports = uarm_ports()
            if len(ports) > 1:
                uarm_port = get_uarm_port_cli()
                try:
                    self.uarm = pyuarm.uArm(uarm_port)
                except pyuarm.UnknownFirmwareException:
                    print ("Unknown Firmware, please use {}{}uarm-firmware -upgrade{}{} to your firmware"
                           .format(Back.BLACK, Fore.WHITE,Fore.RESET, Back.RESET))
            elif len(ports) == 1:
                self.uarm = pyuarm.uArm()
            elif len(ports) == 0:
                print ("No uArm ports is found.")
        else:
            if self.uarm.is_connected():
                print ("uArm is already connected, port: {}".format(self.uarm.port))
            else:
                if self.uarm.reconnect():
                    print ("uArm port: {} is reconnected")

    def do_disconnect(self,arg):
        """
        disconnect, Release uarm port.
        """
        if self.uarm is not None:
            if self.uarm.is_connected():
                self.uarm.disconnect()

    def do_move_to(self, arg):
        """
        move_to, move to destination coordinate.
        format: move_to X Y Z or move_to X Y Z S
        X,Y,Z unit millimeter, S means Speed, unit mm/s
        eg. move_to 100 200 150
        """
        if self.__is_connected():
            values = arg.split(' ')
            if len(values) == 3:
                result = self.uarm.move_to(int(values[0]), int(values[1]), int(values[2]))
                msg = "succeed" if result else "failed"
                print (msg)
            elif len(values) == 4:
                result = self.uarm.move_to(int(values[0]), int(values[1]), int(values[2]), int(values[3]))
                msg = "succeed" if result else "failed"
                print (msg)

    def do_mv(self, args):
        """
        same with move_to
        """
        self.do_move_to(args)

    def do_get_coord(self, arg):
        """
        get_coord get current coordinate
        """
        if self.__is_connected():
            coords = self.uarm.read_coordinate()
            print ("Current coordinate: X-{} Y-{} Z-{}".format(coords.pop(),coords.pop(),coords.pop()))

    def do_pump(self, arg):
        """
        pump
        turn on/off pump
        """
        if self.__is_connected():
            if arg == 'on':
                result = self.uarm.pump_control(True)
                print ("succeed" if result else "failed")
            elif arg == 'off':
                result = self.uarm.pump_control(False)
                print ("succeed" if result else "failed")
            elif arg == '':
                print ("please input argument: {}".format(','.join(self.ON_OFF)))
            else:
                print ("Command not found {}".format(arg))

    def complete_pump(self, text, line, begidx, endidx):
        if not text:
            completions = self.ON_OFF[:]
        else:
            completions = [f
                           for f in self.ON_OFF
                           if f.startswith(text)
                           ]
        return completions

    # def do_sim(self, arg):
    #     """
    #     sim
    #     format: sim X Y Z
    #     validate the coordinate.
    #     eg. sim 100 200 100
    #     succeed
    #     """
    #     if self.__is_connected():
    #         values = arg.split(' ')
    #         if len(values) == 3:
    #             result = self.uarm.get_simulation(int(values[0]), int(values[1]), int(values[2]))
    #             msg = "succeed" if result else "failed"
    #             print (msg)

    def do_gripper(self, arg):
        """
        gripper
        format: gripper on/ gripper off
        """
        if self.__is_connected():
            if arg == 'on':
                result = self.uarm.gripper_control(True)
                print ("succeed" if result else "failed")
            elif arg == 'off':
                result = self.uarm.gripper_control(False)
                print ("succeed" if result else "failed")
            elif arg == '':
                print ("please input argument: {}".format(','.join(self.ON_OFF)))
            else:
                print ("Command not found {}".format(arg))

    def complete_gripper(self, text, line, begidx, endidx):
        if not text:
            completions = self.ON_OFF[:]
        else:
            completions = [f
                           for f in self.ON_OFF
                           if f.startswith(text)
                           ]
        return completions

    def do_write_angle(self, arg):
        """
        write_angle
        format: write_angle servo_number angle
        servo_number:
            0 bottom servo,
            1 left servo,
            2 right servo,
            3 hand servo
        eg.
        >>> write_angle 0 90
        succeed
        """
        if self.__is_connected():
            values = arg.split(' ')
            if len(values) == 2:
                servo_num = int(values[0])
                angle = float(values[1])
                result = self.uarm.write_servo_angle(servo_num, angle)
                msg = "succeed" if result else "failed"
                print (msg)

    def do_read_angle(self, arg):
        """
        read_angle
        Read current servo angle.
        format: read_angle servo_number
        servo_number:
            0 bottom servo,
            1 left servo,
            2 right servo,
        if no servo_number provide, will list all servos angle
        eg.
        >>> read_angle
        Current Servo Angles: b-35.25, l-53.4, r-35.25
        """
        if self.__is_connected():
            values = arg.split(' ')
            if len(values) == 1:
                if values[0] == '':
                    angles = self.uarm.read_servo_angle()
                    print ("Current Servo Angles: b-{}, l-{}, r-{}, h-{}".format(angles.pop(),angles.pop(),angles.pop(), angles.pop()))

                else:
                    servo_num = int(values[0])
                    angle = self.uarm.read_servo_angle(servo_num)
                    print (angle)

    def do_alarm(self, arg):
        """
        alarm
        control buzzer
        format: alarm frequency duration
        eg.
        alarm 1 1
        """
        if self.__is_connected():
            values = arg.split(' ')
            if len(values) == 2:
                start = int(values[1])
                frequency = int(values[0])
                duration = start
                result = self.uarm.alarm(frequency,start,duration)
                msg = "succeed" if result else "failed"
                print (msg)

    def do_set_polar(self, arg):
        if self.__is_connected():
            values = arg.split(' ')
            if len(values) == 2:
                result = self.uarm.write_stretch(values[0], values[1])
                msg = "succeed" if result else "failed"
                print (msg)
            # if len(values) == 3:
            #     result = self.uarm.write_stretch(values[0], values[1], values[2])
            #     msg = "succeed" if result else "failed"
            #     print (msg)

    # def do_get_polar(self, arg):
    #     if self.__is_connected():
    #         result = self.uarm.get_polar()
    #         if result:
    #             print ("polar coordinate: {}".format(result))

    def do_debug(self, arg):
        is_debug = False
        if arg.lower() == 'on':
            is_debug = True
        elif arg.lower() == 'off':
            is_debug = False

        if self.uarm:
            self.uarm.debug = is_debug

    def do_servo(self, arg):
        """
        Servo status
        format: servo attach servo_number
                servo detach servo_number
        servo_number:
            0 bottom servo,
            1 left servo,
            2 right servo,
            3 hand servo
            all
        eg. servo attach all
            servo detach all
        """
        if self.__is_connected():
            values = arg.split(' ')
            if len(values) == 2:
                if values[0] == 'attach':
                    if values[1] == 'all':
                        self.uarm.attach_all_servos()
                    elif values[1].isdigit():
                        v = int(values[1])
                        if 0 <= v <= 3:
                            self.uarm.set_servo_status(v,True)
                elif values[0] == 'detach':
                    if values[1] == 'all':
                        self.uarm.detach_all_servos()
                    elif values[1].isdigit():
                        v = int(values[1])
                        if 0 <= v <= 3:
                            self.uarm.set_servo_status(v,False)

    def complete_servo(self, text, line, begidx, endidx):
        if not text:
            completions = self.SERVO_STATUS[:]
        else:
            completions = [f
                           for f in self.SERVO_STATUS
                           if f.startswith(text)
                           ]
        return completions

    # def do_serial(self, arg):
    #     """
    #     Raw Serial Mode
    #     You could direct input the communication protocol here.
    #     """
    #     if self.__is_connected():
    #         serial_mode = SerialMode(self.uarm)
    #         serial_mode.cmdloop()

    def do_firmware(self, arg):
        """
        Firmware command
        sub command: version, upgrade, force
        version - display Remote Firmware version and local Firmware version (if connected)
        upgrade - if remote version latter than local firmware version, will flash the latest firmware to uarm
        force - force upgrade

        """
        if arg == 'version':
            print ("Remote Firmware Version: {}".format(firmware_helper.get_latest_version()))
            if self.__is_connected():
                print ("Local Firmware Version: {}".format(self.uarm.firmware_version))
        elif arg == 'upgrade':
            if self.uarm is not None:
                if self.uarm.is_connected():
                    self.uarm.disconnect()
                subprocess.call(['uarm-firmware','-u', '-p', self.uarm.port])
            else:
                subprocess.call(['uarm-firmware','-u'])
        elif arg == 'force':
            if self.uarm is not None:
                if self.uarm.is_connected():
                    self.uarm.disconnect()
                subprocess.call(['uarm-firmware','-d', '-f', '-p', self.uarm.port])
            else:
                subprocess.call(['uarm-firmware','-df'])
        elif arg == '':
            print ("please input argument: {}".format(','.join(self.FIRMWARE)))
        else:
            print ("command not found: {}".format(arg))

    def complete_firmware(self, text, line, begidx, endidx):
        if not text:
            completions = self.FIRMWARE[:]
        else:
            completions = [f
                           for f in self.FIRMWARE
                           if f.startswith(text)
                           ]
        return completions

    def do_help(self, arg):
        values = arg.split(' ')
        if len(values) == 1 and values[0] == '':
            help_title = Style.BRIGHT + Fore.MAGENTA + "uArm Command line Help Center" + Fore.RESET + Style.RESET_ALL
            help_title += "\n\n"
            help_title += "Please use {}{}connect{}{} before any control action".format(Back.BLACK, Fore.WHITE, Fore.RESET, Back.RESET)
            help_title += "\n"
            help_title += self.help_msg
            print (help_title)
        # print (self.help_msg)
        Cmd.do_help(self,arg)

    def do_quit(self, args):
        """
        Quit, if uarm is connected, will disconnect before quit
        """
        if self.uarm is not None:
            if self.uarm.is_connected():
                self.uarm.disconnect()
        print ("Quiting")
        raise SystemExit

    do_EOF = do_quit


# class SerialMode(Cmd):
#     prompt = "Serial >>> "
#     intro = "Welcome to Serial Mode."
#     uarm = None
#
#     def __init__(self, uarm):
#         Cmd.__init__(self)
#         self.uarm = uarm
#
#     def default(self, line):
#         if self.uarm.is_connected():
#             response = self.uarm.send_cmd(line)
#             print (response)
#
#     def do_quit(self, args):
#         return True
#
#     do_EOF = do_quit


def main():
    try:
        uarm_cmd = UArmCmd()
        uarm_cmd.cmdloop()
    except KeyboardInterrupt:
        print ("KeyboardInterrupt")

if __name__ == '__main__':
    main()
